@@ -144,7 +144,7 @@ bool StatusLinux::UpdateStatus()
     case IW_MODE_MONITOR:
       type=typeMonitor;
       break;
-#if WIRELESS_EXT>=22
+#if WIRELESS_EXT>=22 && defined(IW_MODE_MESH)
     case IW_MODE_MESH:
       type=typeMesh;
       break;
