@@ -1,5 +1,5 @@
 /*
-  WifiInfo - Show current Wifi sttaus
+  WifiInfo - Show current Wifi status
   Copyright (C) 2004  Tim Teulings
 
   This program is free software; you can redistribute it and/or modify
@@ -17,11 +17,15 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-#include <Lum/OS/Probe.h>
+#include <Lum/Audio/PlaySound.h>
 
 #include <Lum/Base/L10N.h>
+#include <Lum/Base/Path.h>
 #include <Lum/Base/String.h>
 
+#include <Lum/Def/Menu.h>
+#include <Lum/Def/MultiView.h>
+
 #include <Lum/Dlg/About.h>
 
 #include <Lum/Model/Action.h>
@@ -38,10 +42,7 @@
 #include <Lum/FuelGauge.h>
 #include <Lum/Label.h>
 #include <Lum/LED.h>
-#include <Lum/Menu.h>
 #include <Lum/Panel.h>
-#include <Lum/Slider.h>
-#include <Lum/Tab.h>
 #include <Lum/Table.h>
 #include <Lum/Text.h>
 #include <Lum/TextValue.h>
@@ -56,12 +57,93 @@
   #include "StatusNone.h"
 #endif
 
+#include "ConfigDialog.h"
+
 #define NETWORK_REFRESH    10
 #define POLL_ERROR_TIMEOUT 60
 
 #include <iostream>
 static Lum::Def::AppInfo info;
 
+typedef Lum::Model::StdTable<Status::Network> NetworkModel;
+typedef Lum::Base::Reference<NetworkModel>    NetworkModelRef;
+
+class NetworkDataProvider : public NetworkModel::DataProvider
+{
+private:
+  Lum::FuelGauge        *gauge;
+  Lum::Model::DoubleRef data;
+
+public:
+  NetworkDataProvider()
+  : gauge(new Lum::FuelGauge()),
+    data(new Lum::Model::Double())
+  {
+    gauge->SetFlex(true,false);
+
+    data->SetRange(0,100);
+    gauge->SetModel(data);
+  }
+
+  virtual ~NetworkDataProvider()
+  {
+    delete gauge;
+  }
+
+  std::wstring GetString(const NetworkModel::Iterator& iter, size_t column) const
+  {
+    switch (column) {
+    case 1:
+      return iter->essid;
+    case 2:
+        if (iter->channel>=0) {
+          return Lum::Base::NumberToWString(iter->channel);
+        }
+        else {
+          return L"?";
+        }
+    case 3:
+        if (iter->encryption!=0) {
+          std::wstring encryption;
+
+          if (iter->encryption & Status::cryptNone) {
+            encryption.append(L"N");
+          }
+          if (iter->encryption & Status::cryptWEP) {
+            encryption.append(L"W");
+          }
+          if (iter->encryption & Status::cryptWPA_PSK) {
+            encryption.append(L"P");
+          }
+          if (iter->encryption & Status::cryptWPA_EAP) {
+            encryption.append(L"E");
+          }
+          if (iter->encryption & Status::cryptWPA2) {
+            encryption.append(L"2");
+          }
+
+          return encryption;
+        }
+        else {
+          return L"?";
+        }
+    default:
+      return L"";
+    }
+  }
+
+  Lum::Object* GetObject(const NetworkModel::Iterator& iter, size_t column) const
+  {
+    switch (column) {
+    case 4:
+      data->Set(iter->quality);
+      return gauge;
+    default:
+      return NULL;
+    }
+  }
+};
+
 class MyWindow : public Lum::Dialog
 {
 private:
@@ -76,9 +158,10 @@ private:
 
   Lum::Model::IntRef       refreshTime;
 
-  Lum::Model::ListTableRef tableModel;
+  NetworkModelRef          tableModel;
 
   Lum::Model::BooleanRef   scanForNetworks;
+  Lum::Model::BooleanRef   wardrive;
   Lum::Model::ActionRef    powerSavingOnAction;
   Lum::Model::ActionRef    powerSavingOffAction;
 
@@ -87,6 +170,7 @@ private:
   Lum::Model::ActionRef    networksTimer;
   Lum::Model::ActionRef    networksErrorTimer;
 
+  Lum::Model::ActionRef    configAction;
   Lum::Model::ActionRef    aboutAction;
 
 #if defined(CAN_USE_STATUS_HILDON)
@@ -97,6 +181,8 @@ private:
     StatusNone             status;
 #endif
 
+  Lum::Audio::PlaySound    *openNetworkSound;
+
 private:
   Lum::TextValue* CreateTextValue(Lum::Model::StringRef& model, size_t size)
   {
@@ -123,10 +209,15 @@ private:
     }
   }
 
-  void RequestStatus()
+  void CancelStatus()
   {
     Lum::OS::display->RemoveTimer(statusTimer);
     Lum::OS::display->RemoveTimer(statusErrorTimer);
+  }
+
+  void RequestStatus()
+  {
+    CancelStatus();
 
     if (status.UpdateStatus()) {
       Lum::OS::display->AddTimer(refreshTime->Get()+POLL_ERROR_TIMEOUT,0,statusErrorTimer);
@@ -263,71 +354,26 @@ private:
   void UpdateNetworks()
   {
     std::cout << "Updating networks..." << std::endl;
-    std::list<Status::Network>::const_iterator iter;
 
     // First try to update existing entries in the current table
-    iter=status.networks.begin();
-    while (iter!=status.networks.end()) {
+    for (std::list<Status::Network>::const_iterator iter=status.networks.begin();
+         iter!=status.networks.end();
+         ++iter) {
       size_t i;
 
       // Find the right entry
       for (i=1; i<=tableModel->GetRows(); i++) {
-        if (iter->essid==tableModel->GetEntry(i)->GetString(1)) {
+        if (iter->essid==tableModel->GetEntry(i).essid) {
           break;
         }
       }
 
       if (i<=tableModel->GetRows()) { // Found an entry => Update
-        Lum::Model::ListTable::Entry    *entry=tableModel->GetEntry(i);
-        Lum::FuelGauge                  *gauge;
-        Lum::Model::DoubleRef           data;
-
-        gauge=dynamic_cast<Lum::FuelGauge*>(entry->GetObject(4));
-        data=dynamic_cast<Lum::Model::Double*>(gauge->GetModel());
-
-        data->Set(iter->quality);
+        tableModel->GetEntry(i).quality=iter->quality;
       }
       else { // No entry => add
-        Lum::FuelGauge                  *gauge;
-        Lum::Model::DoubleRef           data;
-        Lum::Model::ListTable::StdEntry *entry;
-        std::wstring                    encryption;
-
-        gauge=new Lum::FuelGauge();
-        gauge->SetFlex(true,false);
-
-        data=new Lum::Model::Double();
-        data->SetRange(0,100);
-        data->Set(iter->quality);
-        gauge->SetModel(data);
-
-        entry=new Lum::Model::ListTable::StdEntry(tableModel);
-        entry->SetString(1,iter->essid);
-        entry->SetString(2,Lum::Base::NumberToWString(iter->channel));
-
-        if (iter->encryption & Status::cryptNone) {
-          encryption.append(L"N");
-        }
-        if (iter->encryption & Status::cryptWEP) {
-          encryption.append(L"W");
-        }
-        if (iter->encryption & Status::cryptWPA_PSK) {
-          encryption.append(L"P");
-        }
-        if (iter->encryption & Status::cryptWPA_EAP) {
-          encryption.append(L"E");
-        }
-        if (iter->encryption & Status::cryptWPA2) {
-          encryption.append(L"2");
-        }
-
-        entry->SetString(3,encryption);
-        entry->SetObject(4,gauge);
-
-        tableModel->Append(entry);
+        tableModel->Append(*iter);
       }
-
-      ++iter;
     }
 
     // Now see, if entries in the table have been removed and must
@@ -335,7 +381,7 @@ private:
     size_t i=1;
 
     while(i<=tableModel->GetRows()) {
-      std::wstring                               essid=tableModel->GetEntry(i)->GetString(1);
+      std::wstring                               essid=tableModel->GetEntry(i).essid;
       std::list<Status::Network>::const_iterator iter;
 
       // Find essid from table entry in network list
@@ -356,6 +402,31 @@ private:
       }
     }
     std::cout << "Updating networks done." << std::endl;
+
+    if (wardrive->IsEnabled() && wardrive->Get()) {
+      bool playsound=false;
+
+      std::cout << "Wardriving..." << std::endl;
+
+      for (std::list<Status::Network>::const_iterator iter=status.networks.begin();
+           iter!=status.networks.end();
+           ++iter) {
+        if (iter->quality>=25 && (iter->encryption & Status::cryptNone)) {
+          playsound=true;
+        }
+      }
+
+      if (playsound) {
+        std::cout << "At least one open network with quality>=25% found!" << std::endl;
+        if (openNetworkSound!=NULL) {
+          openNetworkSound->Play();
+        }
+        else {
+          Lum::OS::display->Beep();
+        }
+      }
+      std::cout << "Wardriving done." << std::endl;
+    }
   }
 
 public:
@@ -369,22 +440,27 @@ public:
     channel(new Lum::Model::String()),
     powerSaving(new Lum::Model::Boolean()),
     refreshTime(new Lum::Model::Int(5)),
-    tableModel(new Lum::Model::ListTable()),
+    tableModel(new NetworkModel(new NetworkDataProvider())),
     scanForNetworks(new Lum::Model::Boolean(false)),
+    wardrive(new Lum::Model::Boolean(false)),
     powerSavingOnAction(new Lum::Model::Action()),
     powerSavingOffAction(new Lum::Model::Action()),
     statusTimer(new Lum::Model::Action()),
     statusErrorTimer(new Lum::Model::Action()),
     networksTimer(new Lum::Model::Action()),
     networksErrorTimer(new Lum::Model::Action()),
-    aboutAction(new Lum::Model::Action())
+    configAction(new Lum::Model::Action()),
+    aboutAction(new Lum::Model::Action()),
+    openNetworkSound(NULL)
   {
     quality->SetRange(0,100);
     refreshTime->SetRange(0,10);
 
     powerSaving->Disable();
+    wardrive->Disable();
 
     Observe(scanForNetworks);
+    Observe(wardrive);
     Observe(powerSavingOnAction);
     Observe(powerSavingOffAction);
 
@@ -394,65 +470,71 @@ public:
     Observe(networksTimer);
     Observe(networksErrorTimer);
     Observe(GetOpenedAction());
+    Observe(configAction);
     Observe(aboutAction);
 
     Observe(status.statusChangedAction);
     Observe(status.networksChangedAction);
+
+    Lum::Base::Path soundPath;
+
+    soundPath.SetNativeDir(Lum::Base::Path::GetCWD());
+    soundPath.AppendDir(L"data");
+    soundPath.AppendDir(L"sounds");
+    soundPath.SetBaseName(L"opennetwork.wav");
+
+#if defined(APP_DATADIR)
+    if (!soundPath.Exists()) {
+      soundPath.SetNativeDir(Lum::Base::StringToWString(APP_DATADIR));
+      soundPath.AppendDir(L"sounds");
+      soundPath.SetBaseName(L"opennetwork.wav");
+    }
+#endif
+
+    if (soundPath.Exists()) {
+      openNetworkSound=Lum::Audio::GetPlaySound();
+      if (openNetworkSound!=NULL) {
+        openNetworkSound->SetFilename(soundPath.GetPath(),Lum::Audio::PlaySound::typeWav);
+      }
+    }
+  }
+
+  ~MyWindow()
+  {
+    delete openNetworkSound;
   }
 
   virtual void PreInit()
   {
-    Lum::Label       *label;
-    Lum::Panel       *hPanel,*hPanel2,*vPanel;
-    Lum::Slider      *slider;
-    Lum::Tab         *tab;
+    Lum::Def::MultiView   multiView(Lum::Def::Desc(L"WifiInfo"));
 
-    std::wstring     tmp;
+    Lum::Label            *label;
+    Lum::Panel            *hPanel;
 
-    tab=Lum::Tab::Create(true,true);
-
-    hPanel=Lum::HPanel::Create(true,true);
+    std::wstring          tmp;
 
     label=Lum::Label::Create(true,true);
-    label->AddLabel(L"ESSID:",CreateTextValue(essid,32));
+    label->AddLabel(L"ESSID:",CreateTextValue(essid,28));
     label->AddLabel(L"Channel:",CreateTextValue(channel,2));
-    label->AddLabel(L"Type:",CreateTextValue(type,32));
+    label->AddLabel(L"Type:",CreateTextValue(type,28));
     label->AddLabel(L"AP:",CreateTextValue(accesspoint,17));
     label->AddLabel(L"Client:",CreateTextValue(client,28));
     label->AddLabel(L"Quality:",Lum::FuelGauge::Create(quality,true,false));
     label->AddLabel(L"SNR:",CreateTextValue(signalNoise,18));
 
-    hPanel2=Lum::HPanel::Create();
-    hPanel2->Add(new Lum::LED(powerSaving));
+    hPanel=Lum::HPanel::Create();
+    hPanel->Add(new Lum::LED(powerSaving));
 
     if (status.CanChangePowerSaving()) {
-      hPanel2->AddSpace();
-      hPanel2->Add(Lum::Button::Create(L"_On",powerSavingOnAction));
-      hPanel2->AddSpace();
-      hPanel2->Add(Lum::Button::Create(L"O_ff",powerSavingOffAction));
+      hPanel->AddSpace();
+      hPanel->Add(Lum::Button::Create(L"_On",powerSavingOnAction));
+      hPanel->AddSpace();
+      hPanel->Add(Lum::Button::Create(L"O_ff",powerSavingOffAction));
     }
 
-    label->AddLabel(L"Power saving:",hPanel2);
+    label->AddLabel(L"Power saving:",hPanel);
 
-    hPanel->Add(label);
-
-    hPanel->AddSpace();
-
-    vPanel=Lum::VPanel::Create(false,true);
-    vPanel->Add(Lum::Text::Create(L"Poll (s)"));
-    vPanel->AddSpace();
-
-    slider=new Lum::VSlider();
-    slider->SetFlex(false,true);
-    slider->SetScale(true);
-    slider->SetModel(refreshTime);
-    //label->AddLabel(L"Poll (s):",slider);
-
-    vPanel->Add(slider);
-
-    hPanel->Add(vPanel);
-
-    tab->Add(L"Current",hPanel);
+    multiView.AddView(Lum::Def::MultiView::View(0,Lum::Def::Desc(L"Current"),label));
 
     if (status.SupportsNetworkRetrieval()) {
       Lum::Panel                         *panel;
@@ -461,8 +543,12 @@ public:
       Lum::Model::SingleLineSelectionRef selection;
 
       panel=Lum::VPanel::Create(true,true);
-      panel->Add(Lum::Label::Create(true,false)
-                 ->AddLabel(L"Scan for networks:",Lum::Boolean::Create(scanForNetworks)));
+      panel->Add(Lum::HPanel::Create(true,false)
+                 ->Add(Lum::Label::Create(true,false)
+                        ->AddLabel(L"Scan for networks:",Lum::Boolean::Create(scanForNetworks)))
+                        ->AddSpace()
+                        ->Add(Lum::Label::Create(true,false)
+                              ->AddLabel(L"Wardrive:",Lum::Boolean::Create(wardrive))));
       panel->AddSpace();
 
       selection=new Lum::Model::SingleLineSelection;
@@ -487,22 +573,32 @@ public:
 
       panel->Add(table);
 
-      tab->Add(L"All",panel);
+      multiView.AddView(Lum::Def::MultiView::View(1,Lum::Def::Desc(L"All"),panel));
     }
 
-    SetMain(tab);
+    Lum::Def::Menu *preMenu=Lum::Def::Menu::Create();
 
-    Lum::MenuDesc *menu=new Lum::MenuDesc();
+    preMenu
+      ->GroupProject()
+        ->ActionQuit(GetClosedAction())
+      ->End()
+      ->GroupEdit()
+        ->ActionSettings(configAction)
+      ->End();
 
-    menu
-      ->AddMenuItemSub(_ld(menuProject))
-        ->AddMenuItemAction(_ld(menuProjectQuit),Lum::OS::qualifierControl,L"q",GetClosedAction());
-    menu
-      ->AddMenuItemSub(_ld(menuHelp))
-        ->AddMenuItemAction(_ld(menuHelpHelp),NULL)
-        ->AddMenuItemAction(_ld(menuHelpAbout),aboutAction);
+    Lum::Def::Menu *postMenu=Lum::Def::Menu::Create();
 
-    SetMenu(menu);
+    postMenu
+      ->GroupHelp()
+        //->ActionHelp()
+        ->ActionAbout(aboutAction)
+      ->End();
+
+    //SetMenu(menu);
+
+    multiView.SetMenu(preMenu,postMenu);
+
+    Lum::OS::display->GetBehaviour()->ApplyMultiViewDlg(this,multiView);
 
     Dialog::PreInit();
   }
@@ -517,10 +613,12 @@ public:
     else if (model==scanForNetworks) {
       if (*scanForNetworks) {
         RequestNetworks();
+        wardrive->Enable();
       }
       else {
         CancelNetworks();
         UpdateNetworks();
+        wardrive->Disable();
       }
     }
     else if (model==powerSavingOnAction && powerSavingOnAction->IsFinished()) {
@@ -538,6 +636,7 @@ public:
       RequestNetworks();
     }
     else if (model==refreshTime && IsOpen()) {
+      std::cout << "Refreshing: " << refreshTime->Get() << std::endl;
       RequestStatus();
     }
     else if (model==status.statusChangedAction && status.statusChangedAction->IsFinished()) {
@@ -550,6 +649,18 @@ public:
       UpdateNetworks();
       Lum::OS::display->AddTimer(NETWORK_REFRESH,0,networksTimer);
     }
+    else if (model==configAction && configAction->IsFinished()) {
+      ConfigDialog *dialog;
+
+      CancelStatus();
+      dialog=new ConfigDialog(refreshTime);
+      dialog->SetParent(this);
+      if (dialog->Open()) {
+        dialog->EventLoop();
+        dialog->Close();
+      }
+      RequestStatus();
+    }
     else if (model==aboutAction && aboutAction->IsFinished()) {
       Lum::Dlg::About::Show(this,info);
     }
